@@ -2,17 +2,14 @@
 per-test stdout/stderr capturing mechanism.
 
 """
-from __future__ import absolute_import, division, print_function
-
 import collections
 import contextlib
-import sys
-import os
 import io
+import os
+import sys
 from io import UnsupportedOperation
 from tempfile import TemporaryFile
 
-import six
 import pytest
 from _pytest.compat import CaptureIO
 
@@ -52,13 +49,6 @@ def pytest_load_initial_conftests(early_config, parser, args):
     # make sure that capturemanager is properly reset at final shutdown
     early_config.add_cleanup(capman.stop_global_capturing)
 
-    # make sure logging does not raise exceptions at the end
-    def silence_logging_at_shutdown():
-        if "logging" in sys.modules:
-            sys.modules["logging"].raiseExceptions = False
-
-    early_config.add_cleanup(silence_logging_at_shutdown)
-
     # finally trigger conftest loading but while capturing (issue93)
     capman.start_global_capturing()
     outcome = yield
@@ -69,7 +59,7 @@ def pytest_load_initial_conftests(early_config, parser, args):
         sys.stderr.write(err)
 
 
-class CaptureManager(object):
+class CaptureManager:
     """
     Capture plugin, manages that the appropriate capture method is enabled/disabled during collection and each
     test phase (setup, call, teardown). After each of those points, the captured output is obtained and
@@ -87,6 +77,11 @@ class CaptureManager(object):
         self._global_capturing = None
         self._current_item = None
 
+    def __repr__(self):
+        return "<CaptureManager _method={!r} _global_capturing={!r} _current_item={!r}>".format(
+            self._method, self._global_capturing, self._current_item
+        )
+
     def _getcapture(self, method):
         if method == "fd":
             return MultiCapture(out=True, err=True, Capture=FDCapture)
@@ -94,11 +89,23 @@ class CaptureManager(object):
             return MultiCapture(out=True, err=True, Capture=SysCapture)
         elif method == "no":
             return MultiCapture(out=False, err=False, in_=False)
-        else:
-            raise ValueError("unknown capturing method: %r" % method)
+        raise ValueError("unknown capturing method: %r" % method)  # pragma: no cover
+
+    def is_capturing(self):
+        if self.is_globally_capturing():
+            return "global"
+        capture_fixture = getattr(self._current_item, "_capture_fixture", None)
+        if capture_fixture is not None:
+            return (
+                "fixture %s" % self._current_item._capture_fixture.request.fixturename
+            )
+        return False
 
     # Global capturing control
 
+    def is_globally_capturing(self):
+        return self._method != "no"
+
     def start_global_capturing(self):
         assert self._global_capturing is None
         self._global_capturing = self._getcapture(self._method)
@@ -111,17 +118,29 @@ class CaptureManager(object):
             self._global_capturing = None
 
     def resume_global_capture(self):
-        self._global_capturing.resume_capturing()
+        # During teardown of the python process, and on rare occasions, capture
+        # attributes can be `None` while trying to resume global capture.
+        if self._global_capturing is not None:
+            self._global_capturing.resume_capturing()
 
     def suspend_global_capture(self, in_=False):
         cap = getattr(self, "_global_capturing", None)
         if cap is not None:
             cap.suspend_capturing(in_=in_)
 
+    def suspend(self, in_=False):
+        # Need to undo local capsys-et-al if it exists before disabling global capture.
+        self.suspend_fixture(self._current_item)
+        self.suspend_global_capture(in_)
+
+    def resume(self):
+        self.resume_global_capture()
+        self.resume_fixture(self._current_item)
+
     def read_global_capture(self):
         return self._global_capturing.readouterr()
 
-    # Fixture Control (its just forwarding, think about removing this later)
+    # Fixture Control (it's just forwarding, think about removing this later)
 
     def activate_fixture(self, item):
         """If the current item is using ``capsys`` or ``capfd``, activate them so they take precedence over
@@ -151,15 +170,12 @@ class CaptureManager(object):
 
     @contextlib.contextmanager
     def global_and_fixture_disabled(self):
-        """Context manager to temporarily disables global and current fixture capturing."""
-        # Need to undo local capsys-et-al if exists before disabling global capture
-        self.suspend_fixture(self._current_item)
-        self.suspend_global_capture(in_=False)
+        """Context manager to temporarily disable global and current fixture capturing."""
+        self.suspend()
         try:
             yield
         finally:
-            self.resume_global_capture()
-            self.resume_fixture(self._current_item)
+            self.resume()
 
     @contextlib.contextmanager
     def item_capture(self, when, item):
@@ -237,10 +253,11 @@ def _ensure_only_one_capture_fixture(request, name):
 
 @pytest.fixture
 def capsys(request):
-    """Enable capturing of writes to ``sys.stdout`` and ``sys.stderr`` and make
-    captured output available via ``capsys.readouterr()`` method calls
-    which return a ``(out, err)`` namedtuple.  ``out`` and ``err`` will be ``text``
-    objects.
+    """Enable text capturing of writes to ``sys.stdout`` and ``sys.stderr``.
+
+    The captured output is made available via ``capsys.readouterr()`` method
+    calls, which return a ``(out, err)`` namedtuple.
+    ``out`` and ``err`` will be ``text`` objects.
     """
     _ensure_only_one_capture_fixture(request, "capsys")
     with _install_capture_fixture_on_item(request, SysCapture) as fixture:
@@ -249,26 +266,24 @@ def capsys(request):
 
 @pytest.fixture
 def capsysbinary(request):
-    """Enable capturing of writes to ``sys.stdout`` and ``sys.stderr`` and make
-    captured output available via ``capsys.readouterr()`` method calls
-    which return a ``(out, err)`` tuple.  ``out`` and ``err`` will be ``bytes``
-    objects.
+    """Enable bytes capturing of writes to ``sys.stdout`` and ``sys.stderr``.
+
+    The captured output is made available via ``capsysbinary.readouterr()``
+    method calls, which return a ``(out, err)`` namedtuple.
+    ``out`` and ``err`` will be ``bytes`` objects.
     """
     _ensure_only_one_capture_fixture(request, "capsysbinary")
-    # Currently, the implementation uses the python3 specific `.buffer`
-    # property of CaptureIO.
-    if sys.version_info < (3,):
-        raise request.raiseerror("capsysbinary is only supported on python 3")
     with _install_capture_fixture_on_item(request, SysCaptureBinary) as fixture:
         yield fixture
 
 
 @pytest.fixture
 def capfd(request):
-    """Enable capturing of writes to file descriptors ``1`` and ``2`` and make
-    captured output available via ``capfd.readouterr()`` method calls
-    which return a ``(out, err)`` tuple.  ``out`` and ``err`` will be ``text``
-    objects.
+    """Enable text capturing of writes to file descriptors ``1`` and ``2``.
+
+    The captured output is made available via ``capfd.readouterr()`` method
+    calls, which return a ``(out, err)`` namedtuple.
+    ``out`` and ``err`` will be ``text`` objects.
     """
     _ensure_only_one_capture_fixture(request, "capfd")
     if not hasattr(os, "dup"):
@@ -281,10 +296,11 @@ def capfd(request):
 
 @pytest.fixture
 def capfdbinary(request):
-    """Enable capturing of write to file descriptors 1 and 2 and make
-    captured output available via ``capfdbinary.readouterr`` method calls
-    which return a ``(out, err)`` tuple.  ``out`` and ``err`` will be
-    ``bytes`` objects.
+    """Enable bytes capturing of writes to file descriptors ``1`` and ``2``.
+
+    The captured output is made available via ``capfd.readouterr()`` method
+    calls, which return a ``(out, err)`` namedtuple.
+    ``out`` and ``err`` will be ``byte`` objects.
     """
     _ensure_only_one_capture_fixture(request, "capfdbinary")
     if not hasattr(os, "dup"):
@@ -306,16 +322,16 @@ def _install_capture_fixture_on_item(request, capture_class):
     """
     request.node._capture_fixture = fixture = CaptureFixture(capture_class, request)
     capmanager = request.config.pluginmanager.getplugin("capturemanager")
-    # need to active this fixture right away in case it is being used by another fixture (setup phase)
-    # if this fixture is being used only by a test function (call phase), then we wouldn't need this
-    # activation, but it doesn't hurt
+    # Need to active this fixture right away in case it is being used by another fixture (setup phase).
+    # If this fixture is being used only by a test function (call phase), then we wouldn't need this
+    # activation, but it doesn't hurt.
     capmanager.activate_fixture(request.node)
     yield fixture
     fixture.close()
     del request.node._capture_fixture
 
 
-class CaptureFixture(object):
+class CaptureFixture:
     """
     Object returned by :py:func:`capsys`, :py:func:`capsysbinary`, :py:func:`capfd` and :py:func:`capfdbinary`
     fixtures.
@@ -329,8 +345,7 @@ class CaptureFixture(object):
         self._captured_err = self.captureclass.EMPTY_BUFFER
 
     def _start(self):
-        # Start if not started yet
-        if getattr(self, "_capture", None) is None:
+        if self._capture is None:
             self._capture = MultiCapture(
                 out=True, err=True, in_=False, Capture=self.captureclass
             )
@@ -347,7 +362,7 @@ class CaptureFixture(object):
     def readouterr(self):
         """Read and return the captured output so far, resetting the internal buffer.
 
-        :return: captured content as a namedtuple with  ``out`` and ``err`` string attributes
+        :return: captured content as a namedtuple with ``out`` and ``err`` string attributes
         """
         captured_out, captured_err = self._captured_out, self._captured_err
         if self._capture is not None:
@@ -360,11 +375,13 @@ class CaptureFixture(object):
 
     def _suspend(self):
         """Suspends this fixture's own capturing temporarily."""
-        self._capture.suspend_capturing()
+        if self._capture is not None:
+            self._capture.suspend_capturing()
 
     def _resume(self):
         """Resumes this fixture's own capturing temporarily."""
-        self._capture.resume_capturing()
+        if self._capture is not None:
+            self._capture.resume_capturing()
 
     @contextlib.contextmanager
     def disabled(self):
@@ -393,7 +410,7 @@ def safe_text_dupfile(f, mode, default_encoding="UTF8"):
     return EncodedFile(f, encoding or default_encoding)
 
 
-class EncodedFile(object):
+class EncodedFile:
     errors = "strict"  # possibly needed by py3 code (issue555)
 
     def __init__(self, buffer, encoding):
@@ -401,8 +418,12 @@ class EncodedFile(object):
         self.encoding = encoding
 
     def write(self, obj):
-        if isinstance(obj, six.text_type):
+        if isinstance(obj, str):
             obj = obj.encode(self.encoding, "replace")
+        else:
+            raise TypeError(
+                "write() argument must be str, not {}".format(type(obj).__name__)
+            )
         self.buffer.write(obj)
 
     def writelines(self, linelist):
@@ -414,6 +435,10 @@ class EncodedFile(object):
         """Ensure that file.name is a string."""
         return repr(self.buffer)
 
+    @property
+    def mode(self):
+        return self.buffer.mode.replace("b", "")
+
     def __getattr__(self, name):
         return getattr(object.__getattribute__(self, "buffer"), name)
 
@@ -421,8 +446,9 @@ class EncodedFile(object):
 CaptureResult = collections.namedtuple("CaptureResult", ["out", "err"])
 
 
-class MultiCapture(object):
+class MultiCapture:
     out = err = in_ = None
+    _state = None
 
     def __init__(self, out=True, err=True, in_=True, Capture=None):
         if in_:
@@ -432,7 +458,17 @@ class MultiCapture(object):
         if err:
             self.err = Capture(2)
 
+    def __repr__(self):
+        return "<MultiCapture out={!r} err={!r} in_={!r} _state={!r} _in_suspended={!r}>".format(
+            self.out,
+            self.err,
+            self.in_,
+            self._state,
+            getattr(self, "_in_suspended", "<UNSET>"),
+        )
+
     def start_capturing(self):
+        self._state = "started"
         if self.in_:
             self.in_.start()
         if self.out:
@@ -450,6 +486,7 @@ class MultiCapture(object):
         return out, err
 
     def suspend_capturing(self, in_=False):
+        self._state = "suspended"
         if self.out:
             self.out.suspend()
         if self.err:
@@ -459,6 +496,7 @@ class MultiCapture(object):
             self._in_suspended = True
 
     def resume_capturing(self):
+        self._state = "resumed"
         if self.out:
             self.out.resume()
         if self.err:
@@ -469,9 +507,9 @@ class MultiCapture(object):
 
     def stop_capturing(self):
         """ stop capturing and reset capturing streams """
-        if hasattr(self, "_reset"):
+        if self._state == "stopped":
             raise ValueError("was already stopped")
-        self._reset = True
+        self._state = "stopped"
         if self.out:
             self.out.done()
         if self.err:
@@ -487,18 +525,19 @@ class MultiCapture(object):
         )
 
 
-class NoCapture(object):
+class NoCapture:
     EMPTY_BUFFER = None
     __init__ = start = done = suspend = resume = lambda *args: None
 
 
-class FDCaptureBinary(object):
+class FDCaptureBinary:
     """Capture IO to/from a given os-level filedescriptor.
 
     snap() produces `bytes`
     """
 
-    EMPTY_BUFFER = bytes()
+    EMPTY_BUFFER = b""
+    _state = None
 
     def __init__(self, targetfd, tmpfile=None):
         self.targetfd = targetfd
@@ -508,6 +547,8 @@ class FDCaptureBinary(object):
             self.start = lambda: None
             self.done = lambda: None
         else:
+            self.start = self._start
+            self.done = self._done
             if targetfd == 0:
                 assert not tmpfile, "cannot set tmpfile with stdin"
                 tmpfile = open(os.devnull, "r")
@@ -525,9 +566,11 @@ class FDCaptureBinary(object):
             self.tmpfile_fd = tmpfile.fileno()
 
     def __repr__(self):
-        return "<FDCapture %s oldfd=%s>" % (self.targetfd, self.targetfd_save)
+        return "<FDCapture {} oldfd={} _state={!r}>".format(
+            self.targetfd, getattr(self, "targetfd_save", None), self._state
+        )
 
-    def start(self):
+    def _start(self):
         """ Start capturing on targetfd using memorized tmpfile. """
         try:
             os.fstat(self.targetfd_save)
@@ -535,6 +578,7 @@ class FDCaptureBinary(object):
             raise ValueError("saved filedescriptor not valid anymore")
         os.dup2(self.tmpfile_fd, self.targetfd)
         self.syscapture.start()
+        self._state = "started"
 
     def snap(self):
         self.tmpfile.seek(0)
@@ -543,26 +587,29 @@ class FDCaptureBinary(object):
         self.tmpfile.truncate()
         return res
 
-    def done(self):
+    def _done(self):
         """ stop capturing, restore streams, return original capture file,
         seeked to position zero. """
         targetfd_save = self.__dict__.pop("targetfd_save")
         os.dup2(targetfd_save, self.targetfd)
         os.close(targetfd_save)
         self.syscapture.done()
-        _attempt_to_close_capture_file(self.tmpfile)
+        self.tmpfile.close()
+        self._state = "done"
 
     def suspend(self):
         self.syscapture.suspend()
         os.dup2(self.targetfd_save, self.targetfd)
+        self._state = "suspended"
 
     def resume(self):
         self.syscapture.resume()
         os.dup2(self.tmpfile_fd, self.targetfd)
+        self._state = "resumed"
 
     def writeorg(self, data):
         """ write to original file descriptor. """
-        if isinstance(data, six.text_type):
+        if isinstance(data, str):
             data = data.encode("utf8")  # XXX use encoding of original stream
         os.write(self.targetfd_save, data)
 
@@ -573,19 +620,21 @@ class FDCapture(FDCaptureBinary):
     snap() produces text
     """
 
-    EMPTY_BUFFER = str()
+    # Ignore type because it doesn't match the type in the superclass (bytes).
+    EMPTY_BUFFER = str()  # type: ignore
 
     def snap(self):
-        res = FDCaptureBinary.snap(self)
+        res = super().snap()
         enc = getattr(self.tmpfile, "encoding", None)
         if enc and isinstance(res, bytes):
-            res = six.text_type(res, enc, "replace")
+            res = str(res, enc, "replace")
         return res
 
 
-class SysCapture(object):
+class SysCapture:
 
     EMPTY_BUFFER = str()
+    _state = None
 
     def __init__(self, fd, tmpfile=None):
         name = patchsysdict[fd]
@@ -598,8 +647,14 @@ class SysCapture(object):
                 tmpfile = CaptureIO()
         self.tmpfile = tmpfile
 
+    def __repr__(self):
+        return "<SysCapture {} _old={!r}, tmpfile={!r} _state={!r}>".format(
+            self.name, self._old, self.tmpfile, self._state
+        )
+
     def start(self):
         setattr(sys, self.name, self.tmpfile)
+        self._state = "started"
 
     def snap(self):
         res = self.tmpfile.getvalue()
@@ -610,13 +665,16 @@ class SysCapture(object):
     def done(self):
         setattr(sys, self.name, self._old)
         del self._old
-        _attempt_to_close_capture_file(self.tmpfile)
+        self.tmpfile.close()
+        self._state = "done"
 
     def suspend(self):
         setattr(sys, self.name, self._old)
+        self._state = "suspended"
 
     def resume(self):
         setattr(sys, self.name, self.tmpfile)
+        self._state = "resumed"
 
     def writeorg(self, data):
         self._old.write(data)
@@ -624,7 +682,8 @@ class SysCapture(object):
 
 
 class SysCaptureBinary(SysCapture):
-    EMPTY_BUFFER = bytes()
+    # Ignore type because it doesn't match the type in the superclass (str).
+    EMPTY_BUFFER = b""  # type: ignore
 
     def snap(self):
         res = self.tmpfile.buffer.getvalue()
@@ -633,7 +692,7 @@ class SysCaptureBinary(SysCapture):
         return res
 
 
-class DontReadFromInput(six.Iterator):
+class DontReadFromInput:
     """Temporary stub class.  Ideally when stdin is accessed, the
     capturing should be turned off, with possibly all data captured
     so far sent to the screen.  This should be configurable, though,
@@ -664,10 +723,7 @@ class DontReadFromInput(six.Iterator):
 
     @property
     def buffer(self):
-        if sys.version_info >= (3, 0):
-            return self
-        else:
-            raise AttributeError("redirected stdin has no attribute buffer")
+        return self
 
 
 def _colorama_workaround():
@@ -679,13 +735,11 @@ def _colorama_workaround():
     first import of colorama while I/O capture is active, colorama will
     fail in various ways.
     """
-
-    if not sys.platform.startswith("win32"):
-        return
-    try:
-        import colorama  # noqa
-    except ImportError:
-        pass
+    if sys.platform.startswith("win32"):
+        try:
+            import colorama  # noqa: F401
+        except ImportError:
+            pass
 
 
 def _readline_workaround():
@@ -706,13 +760,11 @@ def _readline_workaround():
 
     See https://github.com/pytest-dev/pytest/pull/1281
     """
-
-    if not sys.platform.startswith("win32"):
-        return
-    try:
-        import readline  # noqa
-    except ImportError:
-        pass
+    if sys.platform.startswith("win32"):
+        try:
+            import readline  # noqa: F401
+        except ImportError:
+            pass
 
 
 def _py36_windowsconsoleio_workaround(stream):
@@ -737,7 +789,11 @@ def _py36_windowsconsoleio_workaround(stream):
 
     See https://github.com/pytest-dev/py/issues/103
     """
-    if not sys.platform.startswith("win32") or sys.version_info[:2] < (3, 6):
+    if (
+        not sys.platform.startswith("win32")
+        or sys.version_info[:2] < (3, 6)
+        or hasattr(sys, "pypy_version_info")
+    ):
         return
 
     # bail out if ``stream`` doesn't seem like a proper ``io`` stream (#2666)
@@ -764,17 +820,6 @@ def _py36_windowsconsoleio_workaround(stream):
             f.line_buffering,
         )
 
-    sys.__stdin__ = sys.stdin = _reopen_stdio(sys.stdin, "rb")
-    sys.__stdout__ = sys.stdout = _reopen_stdio(sys.stdout, "wb")
-    sys.__stderr__ = sys.stderr = _reopen_stdio(sys.stderr, "wb")
-
-
-def _attempt_to_close_capture_file(f):
-    """Suppress IOError when closing the temporary file used for capturing streams in py27 (#2370)"""
-    if six.PY2:
-        try:
-            f.close()
-        except IOError:
-            pass
-    else:
-        f.close()
+    sys.stdin = _reopen_stdio(sys.stdin, "rb")
+    sys.stdout = _reopen_stdio(sys.stdout, "wb")
+    sys.stderr = _reopen_stdio(sys.stderr, "wb")
