@@ -1,12 +1,8 @@
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import os
 import warnings
+from functools import lru_cache
 
 import py
-import six
 
 import _pytest._code
 from _pytest.compat import getfslineno
@@ -18,6 +14,7 @@ SEP = "/"
 tracebackcutdir = py.path.local(_pytest.__file__).dirpath()
 
 
+@lru_cache(maxsize=None)
 def _splitnode(nodeid):
     """Split a nodeid into constituent 'parts'.
 
@@ -35,11 +32,12 @@ def _splitnode(nodeid):
     """
     if nodeid == "":
         # If there is no root node at all, return an empty list so the caller's logic can remain sane
-        return []
+        return ()
     parts = nodeid.split(SEP)
     # Replace single last element 'test_foo.py::Bar' with multiple elements 'test_foo.py', 'Bar'
     parts[-1:] = parts[-1].split("::")
-    return parts
+    # Convert parts into a tuple to avoid possible errors with caching of a mutable type
+    return tuple(parts)
 
 
 def ischildnode(baseid, nodeid):
@@ -54,7 +52,7 @@ def ischildnode(baseid, nodeid):
     return node_parts[: len(base_parts)] == base_parts
 
 
-class Node(object):
+class Node:
     """ base class for Collector and Item the test collection tree.
     Collector subclasses have children, Items are terminal nodes."""
 
@@ -102,7 +100,7 @@ class Node(object):
         return self.session.gethookproxy(self.fspath)
 
     def __repr__(self):
-        return "<%s %s>" % (self.__class__.__name__, getattr(self, "name", None))
+        return "<{} {}>".format(self.__class__.__name__, getattr(self, "name", None))
 
     def warn(self, warning):
         """Issue a warning for this item.
@@ -172,7 +170,7 @@ class Node(object):
         """
         from _pytest.mark import MarkDecorator, MARK_GEN
 
-        if isinstance(marker, six.string_types):
+        if isinstance(marker, str):
             marker = getattr(MARK_GEN, marker)
         elif not isinstance(marker, MarkDecorator):
             raise ValueError("is not a string or pytest.mark.* Marker")
@@ -243,11 +241,10 @@ class Node(object):
     def _repr_failure_py(self, excinfo, style=None):
         if excinfo.errisinstance(fail.Exception):
             if not excinfo.value.pytrace:
-                return six.text_type(excinfo.value)
+                return str(excinfo.value)
         fm = self.session._fixturemanager
         if excinfo.errisinstance(fm.FixtureLookupError):
             return excinfo.value.formatrepr()
-        tbfilter = True
         if self.config.getoption("fulltrace", False):
             style = "long"
         else:
@@ -255,7 +252,6 @@ class Node(object):
             self._prunetraceback(excinfo)
             if len(excinfo.traceback) == 0:
                 excinfo.traceback = tb
-            tbfilter = False  # prunetraceback already does it
             if style == "auto":
                 style = "long"
         # XXX should excinfo.getrepr record all data and toterminal() process it?
@@ -281,11 +277,12 @@ class Node(object):
             abspath=abspath,
             showlocals=self.config.getoption("showlocals", False),
             style=style,
-            tbfilter=tbfilter,
+            tbfilter=False,  # pruned already, or in --fulltrace mode.
             truncate_locals=truncate_locals,
         )
 
-    repr_failure = _repr_failure_py
+    def repr_failure(self, excinfo, style=None):
+        return self._repr_failure_py(excinfo, style)
 
 
 def get_fslocation_from_item(item):
@@ -328,7 +325,7 @@ class Collector(Node):
 
         # Respect explicit tbstyle option, but default to "short"
         # (None._repr_failure_py defaults to "long" without "fulltrace" option).
-        tbstyle = self.config.getoption("tbstyle")
+        tbstyle = self.config.getoption("tbstyle", "auto")
         if tbstyle == "auto":
             tbstyle = "short"
 
@@ -370,9 +367,7 @@ class FSCollector(Collector):
             if nodeid and os.sep != SEP:
                 nodeid = nodeid.replace(os.sep, SEP)
 
-        super(FSCollector, self).__init__(
-            name, parent, config, session, nodeid=nodeid, fspath=fspath
-        )
+        super().__init__(name, parent, config, session, nodeid=nodeid, fspath=fspath)
 
 
 class File(FSCollector):
@@ -387,7 +382,7 @@ class Item(Node):
     nextitem = None
 
     def __init__(self, name, parent=None, config=None, session=None, nodeid=None):
-        super(Item, self).__init__(name, parent, config, session, nodeid=nodeid)
+        super().__init__(name, parent, config, session, nodeid=nodeid)
         self._report_sections = []
 
         #: user properties is a list of tuples (name, value) that holds user
